@@ -9,13 +9,14 @@ from ul_api_utils.api_resource.api_response import JsonApiResponsePayload
 from ul_api_utils.api_resource.api_response_payload_alias import ApiBaseUserModelPayloadResponse, ApiBaseModelPayloadResponse
 from ul_api_utils.internal_api.internal_api import InternalApi
 from ul_api_utils.internal_api.internal_api_response import InternalApiResponse, InternalApiResponseCheckLevel
-from ul_api_utils.validators.custom_fields import WhiteSpaceStrippedStr, PgTypeInt32, CronScheduleStr, UTCOffsetSeconds
+from ul_api_utils.validators.custom_fields import WhiteSpaceStrippedStrAnnotation, PgTypeInt32Annotation, CronScheduleAnnotation, UTCOffsetSecondsAnnotation
 from ul_api_utils.utils.json_encoder import CustomJSONEncoder
 from pydantic import BaseModel, UUID4, Field
 
 from data_aggregator_sdk.constants.enums import IntervalSelectValue, EncryptionType, ResourceKind, NetworkTypeEnum, \
     NetworkSysTypeEnum, DeviceHack, DataAggregatorApiUserType, DownlinkTaskType, ScheduleType, ReglamentType, \
-    SignalModulation, DownlinkTaskStatus, IntegrationV0MessageEvent, JournalDataType, SensorType
+    SignalModulation, DownlinkTaskStatus, IntegrationV0MessageEvent, JournalDataType, SensorType, UnbpGetDataPacket, \
+    UnbpSetSchedulePacket, UnbpSetScheduleAction, DeviceDownlinkTaskStatus, BinaryDataFileType
 from data_aggregator_sdk.data_aggregator_api_sdk_config import DataAggregatorApiSdkConfig
 from data_aggregator_sdk.integration_message import ProfileKind, ProfileGranulation
 from data_aggregator_sdk.types.device import ApiImportDeviceResponse, ApiDataGatewayNetworkDevicePayloadResponse, \
@@ -210,6 +211,10 @@ class GeneralDeviceValueModel(BaseModel):
     devices: List[GeneralDeviceValueDescriptionModel] = Field(title='GeneralDeviceValueDescriptionModels', description='array of GeneralDeviceValueDescriptionModel')
 
 
+class ShortDeviceMeterValuesModel(BaseModel):
+    device_meter_ids: List[UUID] = Field(title='Device Meter ID list', description='array of device meter ids')
+
+
 class GeneralDeviceMeterValueModel(BaseModel):
     devices_meter: List[GeneralDeviceMeterValueDescriptionModel] = Field(title='GeneralDeviceValueDescriptionModels', description='array of GeneralDeviceValueDescriptionModel')
 
@@ -279,6 +284,12 @@ class GeneralApiDeviceMeterValueResponse(JsonApiResponsePayload):
     )
 
 
+class ApiDeviceMeterDailyValueResponse(JsonApiResponsePayload):
+    device_meter_id: UUID = Field(title='Meter Id', description='meter identifier')
+    value_date: datetime = Field(title='Value Date', description='Value Date')
+    value: float = Field(title='Value', description='Value')
+
+
 class ApiDeviceLowBatteryResponse(JsonApiResponsePayload):
     value_date: datetime = Field(title='Value Date', description='Value Date')
     device_id: UUID = Field(title='Device Id', description='device identifier')
@@ -307,7 +318,7 @@ class ApiDeviceNoData(JsonApiResponsePayload):
     kind: ResourceKind = Field(title='Kind', description=f'Kind ({", ".join(kind.name for kind in ResourceKind)})')
     tariff_number: int = Field(title='Tariff Number')
     last_value: Optional[float] = Field(None, title='Last Value')
-    last_value_date: Optional[date] = Field(title='Last Value Date')
+    last_value_date: Optional[date] = Field(None, title='Last Value Date')
     count_days: int = Field(title='Count Days', description='count dates')
 
 
@@ -468,7 +479,7 @@ class SelectValueResultObject(JsonApiResponsePayload):
     last_date: date = Field(title='Last Date', description='last date')
     previous_last_value: float = Field(title='Previous Last Value', description='previous last value')
     current_last_value: float = Field(title='Current Last Value', description='current last value')
-    address_flat_node_id: Optional[str] = Field(title='Address Flat Node Id', description='address flat node id')
+    address_flat_node_id: Optional[str] = Field(None, title='Address Flat Node Id', description='address flat node id')
     mac: Optional[int] = Field(None, title='Mac', description='MAC')
 
 
@@ -484,7 +495,7 @@ class DeviceEventsResponse(JsonApiResponsePayload):
     user_created_id: UUID4 = Field(title='User Created Id', description='user created id')
     value: Optional[float] = Field(None, title='Value', description='device event value')
     device_id: UUID4 = Field(title='Device Id', description='device identifier')
-    serial_number: Optional[int] = Field(title='Serial Number', description='device channel serial number')
+    serial_number: Optional[int] = Field(None, title='Serial Number', description='device channel serial number')
     type: IntegrationV0MessageEvent = Field(title='Type', description='event type')
     date: datetime = Field(title='Date', description='event date')
     data: Optional[Dict[str, Any]] = Field(None, title='Data', description='event extra data')
@@ -493,7 +504,7 @@ class DeviceEventsResponse(JsonApiResponsePayload):
 
 class ApiDeviceStateSyncDataResponse(JsonApiResponsePayload):
     device_events: List[DeviceEventsResponse]
-    offset_id: Optional[UUID]
+    offset_id: Optional[UUID] = None
     has_next: bool
 
 
@@ -517,7 +528,7 @@ class DeviceMeterLastValuesResponse(JsonApiResponsePayload):
 
 class ApiDeviceMeterSyncDataResponse(JsonApiResponsePayload):
     device_meters_values: List[DeviceMeterLastValuesResponse]
-    offset_id: Optional[UUID]
+    offset_id: Optional[UUID] = None
     has_next: bool
 
 
@@ -536,8 +547,20 @@ class ApiLastValueDateResponseForDevices(JsonApiResponsePayload):
 
 
 class ApiDeviceImbalanceModel(BaseModel):
-    flat_devices: Optional[List[List[Any]]] = Field(title='Flat Devices', description='flat devices')
-    group_devices: Optional[List[List[Any]]] = Field(title='Group Devices', description='group devices')
+    flat_devices: Optional[List[List[Any]]] = Field(None, title='Flat Devices', description='flat devices')
+    group_devices: Optional[List[List[Any]]] = Field(None, title='Group Devices', description='group devices')
+
+
+class ApiBinaryDataMetadata(BaseModel):
+    id: UUID4 = Field(title='Id', description='identifier')
+    date_created: datetime = Field(title='Date Created', description='date created')
+    date_modified: datetime = Field(title='Date Modified', description='date modified')
+    user_created_id: UUID4 = Field(title='User Created Id', description='user created identifier')
+    user_modified_id: UUID4 = Field(title='User Modified Id', description='user modified identifier')
+    type: BinaryDataFileType = Field(title='File Type', description='Uploaded file type')
+    description: str = Field(title='Description', description='Uploaded file description')
+    name: str = Field(title='File Name', description='Uploaded file name')
+    file_id: UUID4 = Field(title='File unique ID', description='uuid4')
 
 
 class ApiBinaryDataResponse(JsonApiResponsePayload):
@@ -546,7 +569,36 @@ class ApiBinaryDataResponse(JsonApiResponsePayload):
     user_created_id: UUID4 = Field(title='User Created Id', description='user created id')
     hash: str = Field(title='Hash', description='binary data hash')
     content: str = Field(title='Content', description='binary data content')
-    file_name: str = Field(title='File Name', description='file name')
+    binary_data_metadata: Optional[ApiBinaryDataMetadata] = None
+
+
+class ApiUserResponse(JsonApiResponsePayload):
+    id: UUID4 = Field(title='Api User Identifier', description='unique identifier')
+    date_created: datetime = Field(title='Date Created', description='date created')
+    date_modified: datetime = Field(title='Date Modified', description='date modified')
+    is_alive: bool = Field(title='Is Alive', description='indicator of whether the record is alive')
+    date_expiration: datetime = Field(title='Date Expiration', description='date of user expiration')
+    name: str = Field(title='Name', description='name')
+    note: str = Field(title='Note', description='extra user data')
+    permissions: List[int] = Field(title='Permissions', description='array of permissions')
+
+
+class ApiBaseStationTypeResponse(JsonApiResponsePayload):
+    id: UUID4 = Field(title='Id', description='identifier')
+    date_created: datetime = Field(title='Date Created', description='date created')
+    date_modified: datetime = Field(title='Date Modified', description='date modified')
+    user_created_id: UUID4 = Field(title='User Created Id', description='user created identifier')
+    user_modified_id: UUID4 = Field(title='User Modified Id', description='user modified identifier')
+    name: str = Field(title='Name', description='base station type name')
+    notes: str = Field(title='Notes', description='base station typ notes')
+
+
+class ApiBaseStationResponse(ApiBaseUserModelPayloadResponse):
+    identifier: str = Field(title='Identifier', description='identifier')
+    base_station_api_user_id: UUID = Field(title='Base Station Api User Id', description='base station api user id')
+    base_station_type_id: UUID = Field(title='Base Station Type Id', description='base station type id')
+    base_station_api_user: ApiUserResponse = Field(title='Base Station Api User', description='base station api user')
+    base_station_type: ApiBaseStationTypeResponse = Field(title='Base Station Type', description='base station type')
 
 
 class ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse(JsonApiResponsePayload):
@@ -556,10 +608,10 @@ class ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse(JsonApiResponsePayloa
     uplink_protocol_id: UUID4
     downlink_protocol_id: UUID4
     mac: int
-    key_id: Optional[UUID4]
+    key_id: Optional[UUID4] = None
     device_id: UUID4
-    uplink_encryption_key: Optional[str]
-    downlink_encryption_key: Optional[str]
+    uplink_encryption_key: Optional[str] = None
+    downlink_encryption_key: Optional[str] = None
     uplink_encryption_type: EncryptionType
     downlink_encryption_type: EncryptionType
 
@@ -567,6 +619,8 @@ class ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse(JsonApiResponsePayloa
 class ApiDataGatewayNetworkDeviceShortResponse(JsonApiResponsePayload):
     id: UUID4
     mac: int
+    downlink_encryption_key: Optional[str] = None
+    downlink_encryption_type: EncryptionType
 
 
 class ApiBSUplinkSignalResponse(JsonApiResponsePayload):
@@ -574,16 +628,16 @@ class ApiBSUplinkSignalResponse(JsonApiResponsePayload):
     date_created: datetime
     user_created_id: UUID4
 
-    rssi: Optional[int]
-    snr: Optional[int]
+    rssi: Optional[int] = None
+    snr: Optional[int] = None
     data_gateway_network_device_id: UUID
 
-    data_gateway_network_device: Optional[ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse]
+    data_gateway_network_device: Optional[ApiDataGatewayNetworkDeviceForBsUplinkSignalResponse] = None
     bs_id: UUID
 
 
 class BaseBSDownLinkStatusLogData(BaseModel):
-    effective_date: Optional[datetime]
+    effective_date: Optional[datetime] = None
     bs_downlink_task_id: UUID
     status: DownlinkTaskStatus
     status_comment: str
@@ -596,10 +650,21 @@ class ApiBSDownLinkStatusLogResponse(BaseBSDownLinkStatusLogData, JsonApiRespons
     user_created_id: UUID4
 
 
+class ApiDeviceDownlinkStatusLogResponse(JsonApiResponsePayload):
+    id: UUID4
+    is_active: bool
+    date_created: datetime
+    user_created_id: UUID4
+    effective_date: Optional[datetime] = None
+    bs_downlink_task_data_gateway_network_device_id: UUID
+    status: DeviceDownlinkTaskStatus
+    status_comment: str
+
+
 class BaseApiBSDownlinkTask(JsonApiResponsePayload):
-    note: WhiteSpaceStrippedStr
+    note: WhiteSpaceStrippedStrAnnotation
     type: DownlinkTaskType
-    priority: PgTypeInt32
+    priority: PgTypeInt32Annotation
     broadcast: bool
     schedule_type: ScheduleType
     schedule_effective_date_from: datetime
@@ -613,17 +678,18 @@ class BaseApiBSDownlinkTask(JsonApiResponsePayload):
     lbt_max_waiting_time_ms: int
     lbt_silent_time_ms: int
     force: bool
-    min_tx_delay_after_rx_ms: Optional[int]
-    tx_duration_ms: Optional[int]
+    min_tx_delay_after_rx_ms: Optional[int] = None
+    tx_duration_ms: Optional[int] = None
 
 
 class ApiBSDownlinkTaskTimeSyncResponse(BaseModel):
-    utc_offset_s: UTCOffsetSeconds
+    utc_offset_s: UTCOffsetSecondsAnnotation
 
 
 class ApiBSDownlinkTaskUnbpMessageResponse(BaseModel):
     payload: str
     note: str
+    crc32: Optional[int] = None
 
 
 class ApiBsDownlinkTaskFirmwareUpdateFileResponse(BaseModel):
@@ -645,16 +711,45 @@ class ApiBsDownlinkBaseStationResponse(BaseModel):
     base_station_api_user_id: UUID4
 
 
+class ApiBSDownlinkTaskUnbpMessageSetRelayResponse(BaseModel):
+    state: bool
+    relay_id: int
+
+
+class ApiBSDownlinkTaskUnbpMessageSetScheduleResponse(BaseModel):
+    packet: UnbpSetSchedulePacket
+    action: UnbpSetScheduleAction
+    period_ago: int
+    week_mask: List[int]
+    day_mask: List[int]
+    hour_mask: List[int]
+
+
+class ApiBSDownlinkTaskUnbpMessageSetClockResponse(BaseModel):
+    time: datetime
+    timezone_offset_s: int
+    timezone_offset_is_negative: bool
+
+
+class ApiBSDownlinkTaskUnbpMessageGetDataResponse(BaseModel):
+    date: date
+    request_data_packets: List[UnbpGetDataPacket]
+
+
 class ApiBSDownlinkTaskResponse(BaseApiBSDownlinkTask):
     id: UUID4
     bs: ApiBsDownlinkBaseStationResponse
     date_created: datetime
     user_created_id: UUID4
-    schedule_time: Optional[list[str]]
-    schedule_cron: Optional[list[CronScheduleStr]]
-    bs_downlink_task_time_sync: Optional[ApiBSDownlinkTaskTimeSyncResponse]
-    bs_downlink_task_firmware_update: Optional[ApiBsDownlinkTaskFirmwareUpdateResponse]
-    bs_downlink_task_unbp_message: Optional[ApiBSDownlinkTaskUnbpMessageResponse]
+    schedule_time: Optional[list[str]] = None
+    schedule_cron: Optional[list[CronScheduleAnnotation]] = None
+    bs_downlink_task_time_sync: Optional[ApiBSDownlinkTaskTimeSyncResponse] = None
+    bs_downlink_task_firmware_update: Optional[ApiBsDownlinkTaskFirmwareUpdateResponse] = None
+    bs_downlink_task_unbp_message: Optional[ApiBSDownlinkTaskUnbpMessageResponse] = None
+    bs_downlink_task_unbp_message_set_relay: Optional[ApiBSDownlinkTaskUnbpMessageSetRelayResponse] = None
+    bs_downlink_task_unbp_message_set_schedule: Optional[ApiBSDownlinkTaskUnbpMessageSetScheduleResponse] = None
+    bs_downlink_task_unbp_message_set_clock: Optional[ApiBSDownlinkTaskUnbpMessageSetClockResponse] = None
+    bs_downlink_task_unbp_message_get_data: Optional[ApiBSDownlinkTaskUnbpMessageGetDataResponse] = None
     current_status_log: ApiBSDownLinkStatusLogResponse
     bs_downlink_task_dg_network_devices: List[ApiBsDownlinkTaskDataGatewayNetworkDeviceResponse]
     is_active: Optional[bool] = None
@@ -665,8 +760,8 @@ class ApiDataGatewayNetworkResponse(ApiBaseUserModelPayloadResponse):
     type_network: NetworkTypeEnum
     data_gateway_id: UUID4
     sys_type: NetworkSysTypeEnum
-    specifier: Optional[str]
-    params: Optional[Dict[str, Any]]
+    specifier: Optional[str] = None
+    params: Optional[Dict[str, Any]] = None
 
 
 class DataAggregatorApiUserResponse(ApiBaseModelPayloadResponse):
@@ -685,7 +780,7 @@ class ApiDeviceUptimeResponse(JsonApiResponsePayload):
     user_created_id: UUID4 = Field(title='User Created Identifier', description='api user created id')
 
     device_id: UUID4 = Field(title='Device Identifier', description='device unique identifier')
-    channel_id: UUID4 = Field(title='Device Channel Identifier', description='device channel unique identifier')
+    channel_id: Optional[UUID4] = Field(None, title='Device Channel Identifier', description='device channel unique identifier')
     date: datetime = Field(title='Date', description='date')
     value: int = Field(title='Uptime value', description='uptime value seconds')
 
@@ -695,11 +790,11 @@ class ApiDeviceSensorResponse(JsonApiResponsePayload):
     date_created: datetime = Field(title='Date Created', description='date and time of record creation')
     user_created_id: UUID4 = Field(title='User Created Identifier', description='api user created id')
     device_id: UUID4 = Field(title='Device Identifier', description='device unique identifier')
-    channel_id: Optional[UUID4] = Field(title='Device Channel Identifier', description='device channel unique identifier')
+    channel_id: Optional[UUID4] = Field(None, title='Device Channel Identifier', description='device channel unique identifier')
     date: datetime = Field(title='Date', description='date')
     sensor_type: SensorType = Field(title="Type of Device Sensor", description=f"One of sensor types ({', '.join(sensor_type.name for sensor_type in SensorType)})")
     sensor_id: int = Field(title="Sensor ID", description="Sensor ID")
-    value: Optional[float] = Field(title="Value", description="Device Sensor value")
+    value: Optional[float] = Field(None, title="Value", description="Device Sensor value")
 
 
 class DataAggregatorApiSdk:
@@ -950,14 +1045,31 @@ class DataAggregatorApiSdk:
         iteration_interval: Optional[IntervalSelectValue],
         journal_data_type: Optional[JournalDataType],
         devices_meter: GeneralDeviceMeterValueModel,
+        is_all_tariffs_flag: bool,
+        is_filter_by_period_timezone: bool,
     ) -> List[GeneralApiDeviceMeterValueResponse]:
         return self._api_device.request_post('/list-device/value-for-devices-meter', json=devices_meter, q={
             'period_from': period_from,
             'period_to': period_to,
             'iteration_interval': iteration_interval.value if iteration_interval is not None else None,
             'journal_data_type': journal_data_type.value if journal_data_type is not None else None,
+            'is_all_tariffs_flag': is_all_tariffs_flag,
+            'is_filter_by_period_timezone': is_filter_by_period_timezone,
         }).typed(List[GeneralApiDeviceMeterValueResponse]).check().payload
 
+    '''used in services: Iot Account Application Backend'''
+    @internal_api_error_handler
+    def get_daily_values_value_for_devices_meter(
+        self,
+        value_date: date,
+        device_meter_ids: List[UUID],
+    ) -> List[ApiDeviceMeterDailyValueResponse]:
+        return self._api_device.request_post(
+            '/list-device/value-for-devices-meter/daily-values',
+            json=ShortDeviceMeterValuesModel(device_meter_ids=device_meter_ids),
+            q={'value_date': value_date},
+        ).typed(List[ApiDeviceMeterDailyValueResponse]).check().payload
+
     '''used in services: Iot Account Application Backend'''
     @internal_api_error_handler
     def get_battery_lvl_for_device_list(
@@ -1241,12 +1353,11 @@ class DataAggregatorApiSdk:
     def update_device_meter_sync_offset(self, sync_offset_id: UUID, is_confirmed: bool = True) -> None:
         self._api_device.request_patch(f'/device-meter-sync-offset/{str(sync_offset_id)}', json={'is_confirmed': is_confirmed}).check(level=InternalApiResponseCheckLevel.STATUS_CODE)
 
-
-#     '''used in services: not used'''
-#     @internal_api_error_handler
-#     def get_base_station_by_token(self, api_token: Optional[Tuple[Optional[str], str]]) -> Dict[str, Any]:  # TODO: typing
-#         assert self._api_base_station is not None
-#         return self._api_base_station.request_get('/base-station').check().payload_raw    # type: ignore
+    '''used in services: Service Data Gateway'''
+    @internal_api_error_handler
+    def get_base_station_by_token(self, api_token: str) -> ApiBaseStationResponse:
+        assert self._api_device is not None
+        return self._api_device.request_get('/base-stations/auth-check', access_token=api_token).typed(ApiBaseStationResponse).check().payload
 
     '''used in services: Iot Account Application Backend'''
     @internal_api_error_handler
@@ -1497,7 +1608,7 @@ class DataAggregatorApiSdk:
 #         return self._api_device.request_post(
 #             f'/data-gateways/{gateway_id}/devices',
 #             q=query_params,
-#             json=json.dumps(ApiDataGatewayNetworkDevicesBody(devices=devices).dict()),
+#             json=json.dumps(ApiDataGatewayNetworkDevicesBody(devices=devices).model_dump()),
 #         ).typed(List[ApiDataGatewayNetworkDeviceInfoResponse]).check()
 
     '''used in services: Iot Account Application Gateway, Service Data Gateway'''
