@@ -1,4 +1,4 @@
-from unipipeline.modules.uni import Uni
+from ul_unipipeline.modules.uni import Uni
 
 from data_aggregator_sdk.constants.names import WORKER_NAME
 from data_aggregator_sdk.data_aggregator_sdk_config import DataAggregatorSdkConfig
