@@ -62,31 +62,31 @@ class ApiDeviceModificationTypeResponse(JsonApiResponsePayload):
     name_ru: str
     name_en: str
     type: DeviceModificationTypeEnum
-    metering_type_id: Optional[UUID4]
-    device_metering_type: Optional[ApiDeviceMeteringTypeResponse]
+    metering_type_id: Optional[UUID4] = None
+    device_metering_type: Optional[ApiDeviceMeteringTypeResponse] = None
 
 
 class ApiDeviceModificationResponse(JsonApiResponsePayload):
     id: UUID4
     date_created: datetime
     date_modified: datetime
-    name: Optional[str]
-    device_modification_type_id: Optional[UUID4]
+    name: Optional[str] = None
+    device_modification_type_id: Optional[UUID4] = None
     device_modification_type: ApiDeviceModificationTypeResponse
 
 
 class ApiDeviceChannelPayloadResponse(ApiBaseUserModelPayloadResponse):
     device_id: UUID4
     serial_number: int
-    inactivity_limit: Optional[int]
+    inactivity_limit: Optional[int] = None
     device_meter: List[ApiDeviceMeterPayloadResponse]
 
 
 class ApiDataGatewayNetworkDeviceResponse(ApiBaseUserModelPayloadResponse):
     manufacturer_serial_number: str
-    firmware_version: Optional[str]
-    hardware_version: Optional[str]
-    date_produced: Optional[datetime]
+    firmware_version: Optional[str] = None
+    hardware_version: Optional[str] = None
+    date_produced: Optional[datetime] = None
     device_manufacturer: ApiDeviceManufacturerResponse
     device_modification: ApiDeviceModificationResponse
     device_channel: List[ApiDeviceChannelPayloadResponse]
@@ -98,8 +98,8 @@ class ApiDataGatewaysNetworkResponse(ApiBaseUserModelPayloadResponse):
     data_gateway_id: UUID4
     data_gateway: ApiDataGatewayResponse
     sys_type: NetworkSysTypeEnum
-    specifier: Optional[str]
-    params: Optional[Dict[str, Any]]
+    specifier: Optional[str] = None
+    params: Optional[Dict[str, Any]] = None
 
 
 class ApiDataGatewayNetworkDeviceInfoResponse(JsonApiResponsePayload):
@@ -110,13 +110,13 @@ class ApiDataGatewayNetworkDeviceInfoResponse(JsonApiResponsePayload):
     downlink_protocol_id: UUID4
     data_gateway_network_id: UUID4
     mac: int
-    key_id: Optional[UUID4]
+    key_id: Optional[UUID4] = None
     device_id: UUID4
     device: ApiDataGatewayNetworkDeviceResponse
-    uplink_encryption_key: Optional[str]
-    uplink_decrypted_encryption_key: Optional[str]
-    downlink_encryption_key: Optional[str]
-    downlink_decrypted_encryption_key: Optional[str]
-    encryption_key: Optional[str]
+    uplink_encryption_key: Optional[str] = None
+    uplink_decrypted_encryption_key: Optional[str] = None
+    downlink_encryption_key: Optional[str] = None
+    downlink_decrypted_encryption_key: Optional[str] = None
+    encryption_key: Optional[str] = None
     protocol: ApiProtocolResponse
-    network: Optional[ApiDataGatewaysNetworkResponse]
+    network: Optional[ApiDataGatewaysNetworkResponse] = None
