@@ -1,7 +1,7 @@
 from functools import wraps
 from typing import Any, Callable, Dict, List, TypeVar
 
-from api_utils.errors import Client4XXInternalApiError, Server5XXInternalApiError, \
+from ul_api_utils.errors import Client4XXInternalApiError, Server5XXInternalApiError, \
     ResponseJsonInternalApiError, NotFinishedRequestInternalApiError
 
 from data_aggregator_sdk.errors import DataAggregatorRequestError, DataAggregatorResponseError
